# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from nose.tools import *
import pygraphviz as pgv
from os import linesep

def test_default_attributes():
    A = pgv.AGraph()
    A.graph_attr['label'] = 'test'
    A.graph_attr['spam'] = 'eggs'
    assert_true('label' in A.graph_attr)
    assert_equal(A.graph_attr['label'], 'test')
    assert_equal(A.graph_attr.keys(), ['label', 'spam'])
    assert_equal(sorted(list(A.graph_attr.iteritems())),
                 [('label', 'test'), ('spam', 'eggs')])
    assert_equal(A.string().expandtabs(2),
"""strict graph {
  graph [label=test,
    spam=eggs
  ];
}
""".replace('\n', linesep))

    A.graph_attr['label'] = ''
    A.graph_attr['spam'] = ''

    assert_equal(A.string().expandtabs(2),
"""strict graph {
}
""".replace('\n', linesep))

    A.graph_attr['label'] = 'test'
    del A.graph_attr['label']
    assert_equal(A.string().expandtabs(2),
"""strict graph {
}
""".replace('\n', linesep)
)
def test_graph_defaults():
    A = pgv. AGraph(rankdir='LR',pack='true')
    assert_equal(A.string().expandtabs(2),
"""strict graph {
  graph [pack=true,
    rankdir=LR
  ];
}
""".replace('\n', linesep))


def test_node_defaults():
    A = pgv.AGraph()
    A.node_attr['label']='test'
    assert_true('label' in A.node_attr)
    assert_equal(A.node_attr['label'], 'test')
    assert_equal(A.node_attr.keys(), ['label'])
    assert_equal(A.node_attr, {'label': 'test'})
    assert_equal(list(A.node_attr.iteritems()), [('label', 'test')])
    assert_equal(A.string().expandtabs(2),
"""strict graph {
  node [label=test];
}
""".replace('\n', linesep)
)
    A.node_attr['label'] = ''
    assert_true(A.string().expandtabs(2),
"""strict graph {
}
""".replace('\n', linesep)
)

    A.node_attr['label'] = 'test'
    del A.node_attr['label']
    assert_true(A.string().expandtabs(2),
"""strict graph {
}
""".replace('\n', linesep)
)
    A.graph_attr['fontname'] = 'graph font'
    A.node_attr['fontname'] = 'node font'
    A.edge_attr['fontname'] = 'edge font'
    assert_equal(A.string().expandtabs(2),
"""strict graph {
  graph [fontname="graph font"];
  node [fontname="node font"];
  edge [fontname="edge font"];
}
""".replace('\n', linesep)
)

def test_edge_defaults():
    A = pgv.AGraph()
    A.edge_attr['label'] = 'test'
    assert_true('label' in A.edge_attr)
    assert_equal(A.edge_attr['label'], 'test')
    assert_equal(A.edge_attr.keys(), ['label'])
    assert_equal(A.edge_attr, {'label': 'test'})
    assert_equal(list(A.edge_attr.iteritems()), [('label', 'test')])
    assert_equal(A.string().expandtabs(2),
"""strict graph {
  edge [label=test];
}
""".replace('\n', linesep)
)
    A.edge_attr['label'] = ''
    assert_equal(A.string().expandtabs(2),
"""strict graph {
}
""".replace('\n', linesep)
)
    A.edge_attr['label'] = 'test'
    del A.edge_attr['label']
    assert_true(A.string().expandtabs(2),
"""strict graph {
}
""".replace('\n', linesep)
)



#!/usr/bin/env python
"""
An example that shows how to add your own positions to nodes
and have graphviz "neato" position the edges. 

miles_graph() returns an undirected graph over the 128 US cities from
the datafile miles_dat.txt.  

This example is described in Section 1.1 in Knuth's book [1][2].

The data used in this example is copied from [2].  The filename and
header have been modified to adhere to the request of the author to
not corrupt the original source file content and name.

References.
-----------
[1] Donald E. Knuth,
    "The Stanford GraphBase: A Platform for Combinatorial Computing",
    ACM Press, New York, 1993.
[2] http://www-cs-faculty.stanford.edu/~knuth/sgb.html
"""
from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
import fileinput
__author__ = """Aric Hagberg (aric.hagberg@gmail.com)"""

def miles_graph():
    """Return a graph from the data in miles_dat.txt.

    Edges are made between cities that are less then 300 miles apart.
    """
    import math
    import re
    import gzip

    G=pgv.AGraph(name='miles_dat')
    G.node_attr['shape']='circle'
    G.node_attr['fixedsize']='true'
    G.node_attr['fontsize']='8'
    G.node_attr['style']='filled'
    G.graph_attr['outputorder']='edgesfirst'
    G.graph_attr['label']="miles_dat"
    G.graph_attr['ratio']='1.0'
    G.edge_attr['color']='#1100FF'
    G.edge_attr['style']='setlinewidth(2)'

    cities=[]
    for line in gzip.open("miles_dat.txt.gz",'rt'):
        if line.startswith("*"): # skip comments
            continue
        numfind=re.compile("^\d+") 

        if numfind.match(line): # this line is distances
            dist=line.split()
            for d in dist:
                if float(d) < 300: # connect if closer then 300 miles
                    G.add_edge(city,cities[i])
                i=i+1
        else: # this line is a city, position, population
            i=1
            (city,coordpop)=line.split("[")
            cities.insert(0,city)
            (coord,pop)=coordpop.split("]")
            (y,x)=coord.split(",")
            G.add_node(city)
            n=G.get_node(city)
            # assign positions, scale to be something reasonable in points
            n.attr['pos']="%f,%f)"%(-(float(x)-7000)/10.0,(float(y)-2000)/10.0)
            # assign node size, in sqrt of 1,000,000's of people 
            d=math.sqrt(float(pop)/1000000.0)
            n.attr['height']="%s"%(d/2)
            n.attr['width']="%s"%(d/2)
            # assign node color
            n.attr['fillcolor']="#0000%2x"%(int(d*256))
            # empty labels
            n.attr['label']=' '

    return G

if __name__ == '__main__':
    import warnings
    import pygraphviz as pgv

    # ignore Graphviz warning messages
    warnings.simplefilter('ignore', RuntimeWarning)

    G=miles_graph()
    print("Loaded miles_dat.txt containing 128 cities.")

    G.write("miles.dot")
    print("Wrote miles.dot")
    G.draw("miles.png",prog='neato',args='-n2')
    print("Wrote miles.png")




# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from nose.tools import *
import pygraphviz as pgv
from os import linesep

def test_node_attribute():
    A = pgv.AGraph()
    A.add_node(1,label='test',spam='eggs')
    assert_equal(A.string().expandtabs(2),
"""strict graph {
  node [label="\\N"];
  1  [label=test,
    spam=eggs];
}
""".replace('\n', linesep))

def test_node_attributes2():
    A = pgv.AGraph()
    A.add_node(1)
    one = A.get_node(1)
    one.attr['label'] = 'test'
    one.attr['spam'] = 'eggs'
    assert_true('label' in one.attr)
    assert_equal(one.attr['label'],'test')
    assert_equal(sorted(one.attr.keys()), ['label', 'spam'])
    assert_equal(A.string().expandtabs(2),
"""strict graph {
  node [label="\\N"];
  1  [label=test,
    spam=eggs];
}
""".replace('\n', linesep)
)
    one.attr['label'] = ''
    one.attr['spam'] = ''
    assert_equal(A.string().expandtabs(2),
"""strict graph {
  node [label="\\N"];
  1  [label=""];
}
""".replace('\n', linesep))
    one.attr['label'] = 'test'
    del one.attr['label']
    assert_equal(A.string().expandtabs(2),
"""strict graph {
  node [label="\\N"];
  1  [label=""];
}
""".replace('\n', linesep))

def test_node_attribute_update():
    A = pgv.AGraph()
    A.add_node(1,label='test',spam='eggs')
    A.add_node(1,label='updated')
    assert_equal(A.string().expandtabs(2),
"""strict graph {
  node [label="\\N"];
  1  [label=updated,
    spam=eggs];
}
""".replace('\n', linesep))

def test_node_attribute_remove():
    A = pgv.AGraph()
    A.add_node(1,label='test',spam='eggs')
    A.add_node(1,label=r'\N',spam='') # use \N to signify null label, otherwise ''
    assert_equal(A.string().expandtabs(2),
"""strict graph {
  node [label="\\N"];
  1;
}
""".replace('\n', linesep))



#!/usr/bin/env python

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division

import pygraphviz as pgv
# strict (no parallel edges)
# digraph
# with attribute rankdir set to 'LR'
A=pgv.AGraph(directed=True,strict=True,rankdir='LR')
# add node 1 with color red
A.add_node(1,color='red') 
A.add_node(5,color='blue')
# add some edges
A.add_edge(1,2,color='green')
A.add_edge(2,3)
A.add_edge(1,3)
A.add_edge(3,4)
A.add_edge(3,5)
A.add_edge(3,6)
A.add_edge(4,6)
# adjust a graph parameter
A.graph_attr['epsilon']='0.001'
print(A.string()) # print dot file to standard output
A.layout('dot') # layout with dot
A.draw('foo.ps') # write to file



# -*- coding: utf-8 -*-
from nose.tools import *
import pygraphviz as pgv


def test_del():
    A = pgv.AGraph()
    A.add_node(1,foo='bar')
    # For some reasons after porting to Python 3 clear often cause infinite loop
    A.delete_node('1')
    assert_equal(len(A), 0)

def test_clear_node_with_attributes():
    A = pgv.AGraph()
    A.add_node(1,foo='bar')
    # For some reasons after porting to Python 3 clear often cause infinite loop
    A.clear()
    assert_equal(len(A), 0)
    assert_equal(A.nodes(), [])
    assert_equal(A.node_attr.keys(), [])

def test_clear_graph_attributes():
    A = pgv.AGraph()
    A.add_node(1,foo='bar')
    A.graph_attr.update(landscape='true',ranksep='0.1')
    # For some reasons after porting to Python 3 clear often cause infinite loop
    A.clear()
    assert_equal(len(A), 0)
    assert_equal(A.nodes(), [])
    assert_equal(A.node_attr.keys(), [])
    assert_equal(A.graph_attr.keys(), [])


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Simple example for rendering a graph with the Django web framework.
See
http://www.djangoproject.com/
and
http://www.djangobook.com/en/beta/chapter11/

"""
#    Copyright (C) 2007 by 
#    Aric Hagberg <hagberg@lanl.gov>
#    Dan Schult <dschult@colgate.edu>
#    Manos Renieris, http://www.cs.brown.edu/~er/
#    Distributed with BSD license.     
#    All rights reserved, see LICENSE for details.

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division

__author__ = """Aric Hagberg (hagberg@lanl.gov)"""

try:
    from django.http import HttpResponse
except ImportError: # this won't run without Django, print message 
    print("Django not found.")

def pygraphviz_graph(request):
    import pygraphviz as P
    A=P.AGraph() # init empty graph
    # set some default node attributes
    A.node_attr['style']='filled'
    A.node_attr['shape']='circle'
    # Add edges (and nodes)
    A.add_edge(1,2)
    A.add_edge(2,3)
    A.add_edge(1,3)
    A.layout() # layout with default (neato)
    png=A.draw(format='png') # draw png
    return HttpResponse(png, mimetype='image/png')

if __name__ == '__main__':
    print("""This code works with the Django web framework
and should run as a django "view".
See djangoproject.com for info.

""")



# -*- coding: utf-8 -*-
from nose.tools import *
import pygraphviz as pgv

def test_readwrite():
    A = pgv.AGraph(name='test graph')
    A.add_path([1,2,3,4,5,6,7,8,9,10])

#FIXME
# >>> (fd,fname)=tempfile.mkstemp()
    # A.write(fname)
    # A.read(fname)
    # assert_equal(B, AGraph(fname))
    # assert_true(B == A)
    # assert_false(B is A)
# >>> os.close(fd)
# >>> os.unlink(fname)



# This file was automatically generated by SWIG (http://www.swig.org).
# Version 3.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.





from sys import version_info
if version_info >= (2,6,0):
    def swig_import_helper():
        from os.path import dirname
        import imp
        fp = None
        try:
            fp, pathname, description = imp.find_module('_graphviz', [dirname(__file__)])
        except ImportError:
            import _graphviz
            return _graphviz
        if fp is not None:
            try:
                _mod = imp.load_module('_graphviz', fp, pathname, description)
            finally:
                fp.close()
            return _mod
    _graphviz = swig_import_helper()
    del swig_import_helper
else:
    import _graphviz
del version_info
try:
    _swig_property = property
except NameError:
    pass # Python < 2.2 doesn't have 'property'.
def _swig_setattr_nondynamic(self,class_type,name,value,static=1):
    if (name == "thisown"): return self.this.own(value)
    if (name == "this"):
        if type(value).__name__ == 'SwigPyObject':
            self.__dict__[name] = value
            return
    method = class_type.__swig_setmethods__.get(name,None)
    if method: return method(self,value)
    if (not static):
        self.__dict__[name] = value
    else:
        raise AttributeError("You cannot add attributes to %s" % self)

def _swig_setattr(self,class_type,name,value):
    return _swig_setattr_nondynamic(self,class_type,name,value,0)

def _swig_getattr(self,class_type,name):
    if (name == "thisown"): return self.this.own()
    method = class_type.__swig_getmethods__.get(name,None)
    if method: return method(self)
    raise AttributeError(name)

def _swig_repr(self):
    try: strthis = "proxy of " + self.this.__repr__()
    except: strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)

try:
    _object = object
    _newclass = 1
except AttributeError:
    class _object : pass
    _newclass = 0



def agopen(*args):
  return _graphviz.agopen(*args)
agopen = _graphviz.agopen
def agraphnew(name,strict=False,directed=False):
    if strict:
       if directed:
            return _graphviz.agopen(name,cvar.Agstrictdirected,None)
       else:
            return _graphviz.agopen(name,cvar.Agstrictundirected,None)
    else:
        if directed:
            return _graphviz.agopen(name,cvar.Agdirected,None)
        else:		 
            return _graphviz.agopen(name,cvar.Agundirected,None)


def agclose(*args):
  return _graphviz.agclose(*args)
agclose = _graphviz.agclose

def agread(*args):
  return _graphviz.agread(*args)
agread = _graphviz.agread

def agwrite(*args):
  return _graphviz.agwrite(*args)
agwrite = _graphviz.agwrite

def agisundirected(*args):
  return _graphviz.agisundirected(*args)
agisundirected = _graphviz.agisundirected

def agisdirected(*args):
  return _graphviz.agisdirected(*args)
agisdirected = _graphviz.agisdirected

def agisstrict(*args):
  return _graphviz.agisstrict(*args)
agisstrict = _graphviz.agisstrict

def agnode(*args):
  return _graphviz.agnode(*args)
agnode = _graphviz.agnode

def agidnode(*args):
  return _graphviz.agidnode(*args)
agidnode = _graphviz.agidnode

def agsubnode(*args):
  return _graphviz.agsubnode(*args)
agsubnode = _graphviz.agsubnode

def agfstnode(*args):
  return _graphviz.agfstnode(*args)
agfstnode = _graphviz.agfstnode

def agnxtnode(*args):
  return _graphviz.agnxtnode(*args)
agnxtnode = _graphviz.agnxtnode

def aglstnode(*args):
  return _graphviz.aglstnode(*args)
aglstnode = _graphviz.aglstnode

def agprvnode(*args):
  return _graphviz.agprvnode(*args)
agprvnode = _graphviz.agprvnode

def agedge(*args):
  return _graphviz.agedge(*args)
agedge = _graphviz.agedge

def agidedge(*args):
  return _graphviz.agidedge(*args)
agidedge = _graphviz.agidedge

def agsubedge(*args):
  return _graphviz.agsubedge(*args)
agsubedge = _graphviz.agsubedge

def agfstin(*args):
  return _graphviz.agfstin(*args)
agfstin = _graphviz.agfstin

def agnxtin(*args):
  return _graphviz.agnxtin(*args)
agnxtin = _graphviz.agnxtin

def agfstout(*args):
  return _graphviz.agfstout(*args)
agfstout = _graphviz.agfstout

def agnxtout(*args):
  return _graphviz.agnxtout(*args)
agnxtout = _graphviz.agnxtout

def agfstedge(*args):
  return _graphviz.agfstedge(*args)
agfstedge = _graphviz.agfstedge

def agnxtedge(*args):
  return _graphviz.agnxtedge(*args)
agnxtedge = _graphviz.agnxtedge

def aghead(*args):
  return _graphviz.aghead(*args)
aghead = _graphviz.aghead

def agtail(*args):
  return _graphviz.agtail(*args)
agtail = _graphviz.agtail

def agattr(*args):
  return _graphviz.agattr(*args)
agattr = _graphviz.agattr

def agattrsym(*args):
  return _graphviz.agattrsym(*args)
agattrsym = _graphviz.agattrsym

def agnxtattr(*args):
  return _graphviz.agnxtattr(*args)
agnxtattr = _graphviz.agnxtattr

def agget(*args):
  return _graphviz.agget(*args)
agget = _graphviz.agget

def agxget(*args):
  return _graphviz.agxget(*args)
agxget = _graphviz.agxget

def agset(*args):
  return _graphviz.agset(*args)
agset = _graphviz.agset

def agxset(*args):
  return _graphviz.agxset(*args)
agxset = _graphviz.agxset

def agsafeset(*args):
  return _graphviz.agsafeset(*args)
agsafeset = _graphviz.agsafeset

def agattrname(*args):
  return _graphviz.agattrname(*args)
agattrname = _graphviz.agattrname

def agattrdefval(*args):
  return _graphviz.agattrdefval(*args)
agattrdefval = _graphviz.agattrdefval

def agsafeset_label(*args):
  return _graphviz.agsafeset_label(*args)
agsafeset_label = _graphviz.agsafeset_label

def agattr_label(*args):
  return _graphviz.agattr_label(*args)
agattr_label = _graphviz.agattr_label

def agsubg(*args):
  return _graphviz.agsubg(*args)
agsubg = _graphviz.agsubg

def agfstsubg(*args):
  return _graphviz.agfstsubg(*args)
agfstsubg = _graphviz.agfstsubg

def agnxtsubg(*args):
  return _graphviz.agnxtsubg(*args)
agnxtsubg = _graphviz.agnxtsubg

def agparent(*args):
  return _graphviz.agparent(*args)
agparent = _graphviz.agparent

def agroot(*args):
  return _graphviz.agroot(*args)
agroot = _graphviz.agroot

def agdelsubg(*args):
  return _graphviz.agdelsubg(*args)
agdelsubg = _graphviz.agdelsubg

def agnnodes(*args):
  return _graphviz.agnnodes(*args)
agnnodes = _graphviz.agnnodes

def agnedges(*args):
  return _graphviz.agnedges(*args)
agnedges = _graphviz.agnedges

def agdegree(*args):
  return _graphviz.agdegree(*args)
agdegree = _graphviz.agdegree

def agraphof(*args):
  return _graphviz.agraphof(*args)
agraphof = _graphviz.agraphof

def agnameof(*args):
  return _graphviz.agnameof(*args)
agnameof = _graphviz.agnameof

def agdelnode(*args):
  return _graphviz.agdelnode(*args)
agdelnode = _graphviz.agdelnode

def agdeledge(*args):
  return _graphviz.agdeledge(*args)
agdeledge = _graphviz.agdeledge
def agnameof(handle):
  name=_graphviz.agnameof(handle)
  if name is None:
     return None
  if name==b'' or name.startswith(b'%'):
    return None
  else:
    return name 

AGRAPH = _graphviz.AGRAPH
AGNODE = _graphviz.AGNODE
AGOUTEDGE = _graphviz.AGOUTEDGE
AGINEDGE = _graphviz.AGINEDGE
AGEDGE = _graphviz.AGEDGE
# This file is compatible with both classic and new-style classes.

cvar = _graphviz.cvar
Agdirected = cvar.Agdirected
Agstrictdirected = cvar.Agstrictdirected
Agundirected = cvar.Agundirected
Agstrictundirected = cvar.Agstrictundirected




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Create and draw a star with varying node properties.

"""
#    Copyright (C) 2006 by 
#    Aric Hagberg <hagberg@lanl.gov>
#    Dan Schult <dschult@colgate.edu>
#    Manos Renieris, http://www.cs.brown.edu/~er/
#    Distributed with BSD license.     
#    All rights reserved, see LICENSE for details.

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division

__author__ = """Aric Hagberg (hagberg@lanl.gov)"""

from pygraphviz import *

A=AGraph()

# set some default node attributes
A.node_attr['style']='filled'
A.node_attr['shape']='circle'
A.node_attr['fixedsize']='true'
A.node_attr['fontcolor']='#FFFFFF'

# make a star in shades of red
for i in range(16):
    A.add_edge(0,i)
    n=A.get_node(i)
    n.attr['fillcolor']="#%2x0000"%(i*16)
    n.attr['height']="%s"%(i/16.0+0.5)
    n.attr['width']="%s"%(i/16.0+0.5)

print(A.string()) # print to screen
A.write("star.dot") # write to simple.dot
print("Wrote star.dot")
A.draw('star.png',prog="circo") # draw to png using circo
print("Wrote star.png")



# -*- coding: utf-8 -*-
from nose.tools import *
import pygraphviz as pgv
from os import linesep

def test_subgraph():
    G = pgv.AGraph()
    G.add_edge(1,2)
    s = G.add_subgraph((1,2),label='foo')
    assert_equal(G.string().expandtabs(2),
"""strict graph {
  {
    graph [label=foo];
    1 -- 2;
  }
}
""".replace('\n', linesep))



def test_subgraph_cluster():
    G = pgv.AGraph(label='foo')
    s = G.subgraph('cluster_a', label='<Hello<BR/>World>')
    s.add_node('sa')
    G.add_node('a')
    assert_equal(G.string().expandtabs(2),
"""strict graph {
  graph [label=foo];
  {
    graph [label=<Hello<BR/>World>];
    sa;
  }
  a;
}
""".replace('\n', linesep))

def test_subgraph_cluster_attribute():
    G = pgv.AGraph()
    s = G.subgraph(name='cluster_a')
    s.node_attr['foo']='bar'
    G.add_node('a')
    G.node_attr['foo']='baz'
    assert_equal(G.string().expandtabs(2),
"""strict graph {
  node [foo=baz];
  subgraph cluster_a {
    graph [foo=bar];
  }
  a;
}
""".replace('\n', linesep))
